from boa.builtins import verify_signature


def Main(operation):

    if operation == 1:

        # this should return true
        message = 'abcdef'
        signature = b'\xcd\x0c\xa9g\xd1\x1c\xeax\xe2Z\xd1o\x15\xdb\xe7vr%\x8b\xfe\xc5\x9f\xf3a|\x95\xe3\x17\xac\xff\x06:H\xd3_q\xaa\\\xe7\xd75\x97t\x12\x18n\x15rP}\x0fM L[\xcbl\x90\xe0;\x8b\x85\x7f\xbd'
        pubkey = b'\x03o\xbc\xb5\xe18\xc1\xceS`\xe8agL\x03"\x8a\xf75\xa9\x11J[\x7f\xb4\x12\x1b\x83P\x12\x9f?\xfe'

        result = verify_signature(pubkey, signature, message)

        return result

    elif operation == 2:
        # this should return false
        message = 'abcdefg'
        signature = b'\xcd\x0c\xa9g\xd1\x1c\xeax\xe2Z\xd1o\x15\xdb\xe7vr%\x8b\xfe\xc5\x9f\xf3a|\x95\xe3\x17\xac\xff\x06:H\xd3_q\xaa\\\xe7\xd75\x97t\x12\x18n\x15rP}\x0fM L[\xcbl\x90\xe0;\x8b\x85\x7f\xbd'
        pubkey = b'\x03o\xbc\xb5\xe18\xc1\xceS`\xe8agL\x03"\x8a\xf75\xa9\x11J[\x7f\xb4\x12\x1b\x83P\x12\x9f?\xfe'

        result = verify_signature(pubkey, signature, message)

        return result

    return False
